//-------------------------------------------------------------------------------------------------
/// \file	SimpleNode.h
/// \date	6-11-04
/// \author	Rob Bateman
/// \brief
///
//-------------------------------------------------------------------------------------------------

#ifndef __SIMPLE_NODE__H__
#define __SIMPLE_NODE__H__

	#ifdef WIN32
		#define NT_PLUGIN
		#pragma once
		#define WIN32_LEAN_AND_MEAN
		#include <windows.h>
	#endif

	#include <maya/MString.h>
	#include <maya/MGlobal.h>
	#include <maya/MFnDependencyNode.h>
	#include <maya/MTime.h>
	#include <maya/MPlug.h>
	#include <maya/MDagModifier.h>
	#include <maya/MDataBlock.h>
	#include <maya/MDataHandle.h>
	#include <maya/MFnNumericAttribute.h>
	#include <maya/MFnUnitAttribute.h>
	#include <maya/MFloatVector.h>
	#include <maya/MPxNode.h>
	#include <maya/MSelectionList.h>

	//---------------------------------------------------------------------------------------------
	/// \brief	This is the simplest possible node type available. Put simply, it creates a single
	///			node that takes an input connection from time1.outTime to the input time value.
	/// 		A single output attribute then computes the sine of that value
	///
	///
	class SimpleNode
		: public MPxNode
	{
	public:

		/// \brief	The compute function is called to calculate the value of a specific
		/// 		output attribute. This usually only happens when an input value changes
		/// 		on which the output was dependant on; or when the node is forced to be
		/// 		recomputed because it has been flagged as dirty.
		/// \param	plug	-	the output attribute to calculate
		/// \param	data	-	the data from which you MUST read the attributes back from.
		///	\return	Did it succeed or fail?
		///
		/// \note	YOU MUST ONLY EVER READ THE NODE'S ATTRIBUTES BACK FROM THE DATA BLOCK
		/// 		GIVEN TO YOU. YOU MUST NOT ATTEMPT TO USE ANY FUNCTION SETS, AND YOU MUST
		/// 		NOT ACCESS ANY OTHER NODES WHILST IN THE COMPUTE FUNCTION.
		///
		/// 		THE COMPUTE FUNCTION IS THE ONLY FUNCTION ON OUR NODE, THAT IS LIKELY TO
		/// 		BE USED DURING RENDERING. BY ACCESSING ONLY THE ATTRIBUTES VIA THE DATA
		/// 		BLOCK, MAYA ENSURES THAT RENDERING REMAINS THREAD SAFE. DUPLICATE
		/// 		DATA BLOCKS CAN BE PASSED TO TWO OR MORE PROCESSORS. THIS MEANS THAT NO
		/// 		SHARED DATA WILL BE ACCESSED BY DIFFERENT THREADS. ie, EACH THREAD RECIEVES
		/// 		IT'S OWN COPY OF THE DATA.
		///
		/// 		IF YOU DO START ACCESSING GLOBAL DATA, OR DATA FROM OTHER NODES WITHIN THE
		/// 		COMPUTE FUNCTION - IT WILL PROBABLY WORK WHILST YOU'RE WORKING INSIDE MAYA,
		/// 		HOWEVER - IT IS LIABLE TO CRASH WHEN RENDERING, ie WHEN MULTIPLE PROCESSORS
		/// 		ARE USED.
		///
		virtual MStatus compute( const MPlug&, MDataBlock& );

	public:

		/// \brief	This function returns a new instance of our node
		/// \return	A new node instance
		///
		static  void *  creator();

		/// \brief	This function sets up the attributes on the node
		/// \return	ok or fail
		///
		static  MStatus initialize();

	public:

		// 	------------ The Type Info ------------

		/// The specific ID for this node type
		const static  MTypeId m_TypeId;

		/// The type name for this new node type
		const static  MString m_TypeName;

	private:

		// 	------------ The attributes ------------

		/// an connection made to the input time value
		static MObject m_aTime;

		/// A single output floating point value
		static MObject m_aOutput;


		static MObject m_aScalar;

	};


#endif
